"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.packageJson = exports.GIT_COMMIT_HASH = void 0;
exports.GIT_COMMIT_HASH = '532eec05f4061c51d428d8c8efee9a43da76a611';
exports.packageJson = {
    "name": "wechaty",
    "version": "1.13.21",
    "description": "Wechaty is a RPA SDK for Chatbot Makers.",
    "type": "module",
    "exports": {
        ".": {
            "import": "./dist/esm/src/mods/mod.js",
            "require": "./dist/cjs/src/mods/mod.js"
        },
        "./impls": {
            "import": "./dist/esm/src/mods/impls.js",
            "require": "./dist/cjs/src/mods/impls.js"
        },
        "./users": {
            "import": "./dist/esm/src/mods/users.js",
            "require": "./dist/cjs/src/mods/users.js"
        }
    },
    "typesVersions": {
        "*": {
            "users": [
                "./dist/esm/src/mods/users.d.ts"
            ],
            "impls": [
                "./dist/esm/src/mods/impls.d.ts"
            ]
        }
    },
    "types": "./dist/esm/src/mods/mod.d.ts",
    "engines": {
        "node": ">=16",
        "npm": ">=7"
    },
    "wechaty": {
        "DEFAULT_PORT": 8080,
        "DEFAULT_PROTOCOL": "io|0.0.1",
        "DEFAULT_APIHOST": "api.chatie.io"
    },
    "scripts": {
        "build": "tsc && tsc -p tsconfig.cjs.json",
        "changelog": "docker run -it --rm -e CHANGELOG_GITHUB_TOKEN -v \"$(pwd)\":/usr/local/src/your-app ferrarimarco/github-changelog-generator -u wechaty -p wechaty && sed -i'.bak' /greenkeeper/d CHANGELOG.md && sed -i'.bak' /Snyk/d CHANGELOG.md && sed -i'.bak' '/An in-range update of/d' CHANGELOG.md && node --no-warnings --loader=ts-node/esm scripts/sort-contributiveness.ts < CHANGELOG.md > CHANGELOG.new.md 2>/dev/null && cat CHANGELOG.md >> CHANGELOG.new.md && mv CHANGELOG.new.md CHANGELOG.md",
        "clean": "shx rm -fr dist/*",
        "coverage": "nyc report --reporter=lcov",
        "cli": "node --no-warnings --loader ts-node/esm bin/cli.ts",
        "demo": "cross-env NODE_OPTIONS=\"--no-warnings --loader=ts-node/esm\" node examples/ding-dong-bot.ts",
        "dist": "npm-run-all clean build dist:commonjs",
        "dist:commonjs": "jq -n \"{ type: \\\"commonjs\\\" }\" > dist/cjs/package.json",
        "docs": "bash -x scripts/generate-docs.sh",
        "doctor": "node --no-warnings --loader=ts-node/esm bin/doctor.ts",
        "io-client": "node --no-warnings --loader=ts-node/esm bin/io-client.ts",
        "lint:es": "eslint --ignore-pattern node_modules/ --ignore-pattern fixtures/ \"{bin,examples,src,scripts,tests}/**/*.ts\"",
        "lint:md": "markdownlint README.md",
        "lint:sh": "bash -n bin/*.sh",
        "lint:ts": "tsc --isolatedModules --noEmit",
        "lint": "npm-run-all lint:es lint:ts lint:sh lint:md",
        "puppet-install": "node --no-warnings --loader=ts-node/esm bin/puppet-install.ts",
        "sloc": "sloc bin examples scripts src tests --details --format cli-table --keys total,source,comment && sloc bin examples scripts src tests",
        "start": "npm run demo",
        "test:pack": "bash -x scripts/npm-pack-testing.sh",
        "test:shell": "shellcheck bin/*.sh scripts/*.sh",
        "test:unit": "cross-env NODE_OPTIONS=\"--no-warnings --loader=ts-node/esm\" TAP_TIMEOUT=600 tap \"src/**/*.spec.ts\" \"tests/**/*.spec.ts\"",
        "test": "npm run lint && nyc npm run test:unit",
        "typedoc": "bash scripts/typedoc.sh"
    },
    "repository": {
        "type": "git",
        "url": "git+https://github.com/wechaty/wechaty.git"
    },
    "bin": {
        "wechaty": "dist/esm/bin/cli.js",
        "wechaty-doctor": "dist/esm/bin/doctor.js",
        "wechaty-io-client": "dist/esm/bin/io-client.js",
        "wechaty-puppet-install": "dist/esm/bin/puppet-install.js",
        "wechaty-version": "dist/esm/bin/version.js"
    },
    "keywords": [
        "ai",
        "bot",
        "chatbot",
        "chatie",
        "conversational",
        "framework",
        "robotics",
        "robot",
        "rpa",
        "sdk",
        "wechat",
        "wecom",
        "wxwork",
        "lark",
        "TikTok",
        "weixin",
        "whatsapp",
        "微信"
    ],
    "author": {
        "name": "Huan LI (李卓桓)",
        "email": "zixia@zixia.net",
        "url": "https://linkedin.com/in/zixia/"
    },
    "license": "Apache-2.0",
    "bugs": {
        "url": "https://github.com/wechaty/wechaty/issues"
    },
    "funding": {
        "type": "opencollective",
        "url": "https://opencollective.com/wechaty"
    },
    "homepage": "https://github.com/wechaty/",
    "dependencies": {
        "clone-class": "^1.1.1",
        "cmd-ts": "^0.9.0",
        "cockatiel": "^2.0.2",
        "cross-spawn": "^7.0.3",
        "dotenv": "^10.0.0",
        "file-box": "^1.4.15",
        "fp-ts": "^2.11.5",
        "gerror": "^1.0.16",
        "get-port": "^6.0.0",
        "json-rpc-peer": "^0.17.0",
        "memory-card": "^1.1.2",
        "open-graph": "^0.2.6",
        "rx-queue": "^1.0.4",
        "state-switch": "^1.6.3",
        "uuid": "^8.3.2",
        "wechaty-puppet": "~1.13.8",
        "wechaty-puppet-service": "~1.11.2",
        "wechaty-puppet-wechat4u": "~1.11.1",
        "wechaty-token": "^1.1.1",
        "ws": "^8.4.0"
    },
    "devDependencies": {
        "@chatie/eslint-config": "^1.0.4",
        "@chatie/git-scripts": "^0.6.2",
        "@chatie/semver": "^0.4.7",
        "@chatie/tsconfig": "^4.6.2",
        "@swc/core": "^1.2.124",
        "@swc/helpers": "^0.3.2",
        "@types/cross-spawn": "^6.0.2",
        "@types/dotenv": "^8.2.0",
        "@types/glob": "^7.2.0",
        "@types/open-graph": "^0.2.2",
        "@types/qrcode-terminal": "^0.12.0",
        "@types/raven": "^2.5.4",
        "@types/uuid": "^8.3.3",
        "@types/ws": "^8.2.2",
        "coveralls": "^3.1.1",
        "glob": "^7.2.0",
        "jsdoc-to-markdown": "^7.1.0",
        "nyc": "^15.1.0",
        "qrcode-terminal": "^0.12.0",
        "raven": "^2.6.4",
        "regenerator-runtime": "^0.13.9",
        "sloc": "^0.2.1",
        "typed-emitter": "^1.5.0-from-event",
        "typedoc": "^1.0.0-dev.4 ",
        "wechaty-puppet-mock": "^1.10.2",
        "wechaty-puppet-padlocal": "^0.4.2"
    },
    "files": [
        "bin/",
        "dist/",
        "src/"
    ],
    "publishConfig": {
        "access": "public",
        "tag": "next"
    },
    "collective": {
        "type": "opencollective",
        "url": "https://opencollective.com/wechaty"
    },
    "git": {
        "scripts": {
            "pre-push": "npx git-scripts-pre-push"
        }
    }
};
//# sourceMappingURL=package-json.js.map