#!/usr/bin/env -S node --no-warnings --loader ts-node/esm
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *   Wechaty Chatbot SDK - https://github.com/wechaty/wechaty
 *
 *   @copyright 2016 Huan LI (李卓桓) <https://github.com/huan>, and
 *                   Wechaty Contributors <https://github.com/wechaty>.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
const tstest_1 = require("tstest");
const memory_card_1 = require("memory-card");
const wechaty_puppet_mock_1 = require("wechaty-puppet-mock");
/**
 * Huan(202111): must import `./wechaty-impl.js`
 *  before import `./wechaty-base.js`
 *
 * Or will throw error:
 *
 *    ReferenceError: Cannot access 'WechatyBase' before initialization
 *    at file:///home/huan/git/wechaty/wechaty/src/wechaty/wechaty-impl.ts:25:47
 *
 * TODO: find out why
 */
require("./wechaty-impl.js");
const wechaty_base_js_1 = require("./wechaty-base.js");
const wechaty_skeleton_js_1 = require("./wechaty-skeleton.js");
class WechatyTest extends wechaty_base_js_1.WechatyBase {
}
(0, tstest_1.test)('static VERSION', async (t) => {
    t.ok('VERSION' in wechaty_base_js_1.WechatyBase, 'Wechaty should has a static VERSION property');
});
(0, tstest_1.test)('event:start/stop', async (t) => {
    const wechaty = new wechaty_base_js_1.WechatyBase({ puppet: 'wechaty-puppet-mock' });
    const startSpy = tstest_1.sinon.spy();
    const stopSpy = tstest_1.sinon.spy();
    wechaty.on('start', startSpy);
    wechaty.on('stop', stopSpy);
    await wechaty.start();
    await wechaty.stop();
    // console.log(startSpy.callCount)
    t.ok(startSpy.calledOnce, 'should get event:start once');
    t.ok(stopSpy.calledOnce, 'should get event:stop once');
});
//
// FIXME: restore this unit test !!!
//
// test.only('event:scan', async t => {
//   const m = {} as any
//   const asyncHook = asyncHooks.createHook({
//     init(asyncId: number, type: string, triggerAsyncId: number, resource: Object) {
//       m[asyncId] = type
//     },
//     before(asyncId) {
//       // delete m[asyncId]
//     },
//     after(asyncId) {
//       // delete m[asyncId]
//     },
//     destroy(asyncId) {
//       delete m[asyncId]
//     },
//   })
//   asyncHook.enable()
//   const wechaty = Wechaty.instance()
//   const spy = sinon.spy()
//   wechaty.on('scan', spy)
//   const scanFuture  = new Promise(resolve => wechaty.once('scan', resolve))
//   // wechaty.once('scan', () => console.log('FAINT'))
//   await wechaty.start()
//   await scanFuture
//   // await new Promise(r => setTimeout(r, 1000))
//   await wechaty.stop()
//   t.ok(spy.calledOnce, 'should get event:scan')
//   asyncHook.disable()
//   console.log(m)
// })
tstest_1.test.skip('SKIP DEALING WITH THE LISTENER EXCEPTIONS. on(event, Function)', async (t) => {
    const spy = tstest_1.sinon.spy();
    const wechaty = new wechaty_base_js_1.WechatyBase();
    const EXPECTED_ERROR = new Error('testing123');
    wechaty.on('message', () => { throw EXPECTED_ERROR; });
    // wechaty.on('scan',    () => 42)
    wechaty.on('error', spy);
    const messageFuture = new Promise(resolve => wechaty.once('message', resolve));
    wechaty.emit('message', {});
    await messageFuture;
    await wechaty.stop();
    t.ok(spy.calledOnce, 'should get event:error once');
    t.equal(spy.firstCall.args[0], EXPECTED_ERROR, 'should get error from message listener');
});
tstest_1.test.skip('SKIP DEALING WITH THE LISTENER EXCEPTIONS. test async error', async (t) => {
    // Do not modify the global Wechaty instance
    class MyWechatyTest extends wechaty_base_js_1.WechatyBase {
    }
    const EXPECTED_ERROR = new Error('test');
    const bot = new MyWechatyTest({
        puppet: new wechaty_puppet_mock_1.PuppetMock(),
    });
    const asyncErrorFunction = function () {
        return new Promise((resolve, reject) => {
            setTimeout(function () {
                reject(EXPECTED_ERROR);
            }, 100);
            // tslint ask resolve must be called,
            // so write a falsy value, so that it never called
            if (+new Date() < 0) {
                resolve();
            }
        });
    };
    bot.on('message', async () => {
        await asyncErrorFunction();
    });
    bot.on('error', (e) => {
        t.ok(e.message === EXPECTED_ERROR.message);
    });
    bot.emit('message', {});
    await bot.stop();
});
(0, tstest_1.test)('use plugin', async (t) => {
    // Do not modify the gloabl Wechaty instance
    class MyWechatyTest extends wechaty_base_js_1.WechatyBase {
    }
    let result = '';
    const myGlobalPlugin = function () {
        return function (bot) {
            bot.on('message', () => { result += 'FROM_GLOBAL_PLUGIN:'; });
        };
    };
    const myPlugin = function () {
        return function (bot) {
            bot.on('message', () => { result += 'FROM_MY_PLUGIN:'; });
        };
    };
    MyWechatyTest.use(myGlobalPlugin());
    const bot = new MyWechatyTest({
        puppet: new wechaty_puppet_mock_1.PuppetMock(),
    });
    bot.use(myPlugin());
    await bot.start();
    bot.on('message', () => (result += 'FROM_BOT'));
    bot.emit('message', {});
    await bot.stop();
    t.equal(result, 'FROM_GLOBAL_PLUGIN:FROM_MY_PLUGIN:FROM_BOT', 'should get plugin works');
});
(0, tstest_1.test)('wechatifyUserModules()', async (t) => {
    const wechatyTest = new WechatyTest();
    t.doesNotThrow(() => wechatyTest.__wechatifyUserModules(), 'should not throw for the 1st time init');
    t.doesNotThrow(() => wechatyTest.__wechatifyUserModules(), 'should not throw for the 2nd time init (silence skip)');
});
// TODO: add test for event args
(0, tstest_1.test)('Perfect restart', async (t) => {
    const wechaty = new wechaty_base_js_1.WechatyBase({
        puppet: new wechaty_puppet_mock_1.PuppetMock(),
    });
    try {
        for (let i = 0; i < 3; i++) {
            await wechaty.start();
            await wechaty.stop();
            t.pass('start/stop-ed at #' + i);
        }
        t.pass('Wechaty start/restart successed.');
    }
    catch (e) {
        t.fail(e);
    }
});
(0, tstest_1.test)('@event ready', async (t) => {
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const wechaty = new wechaty_base_js_1.WechatyBase({ puppet });
    const sandbox = tstest_1.sinon.createSandbox();
    const spy = sandbox.spy();
    wechaty.on('ready', spy);
    t.ok(spy.notCalled, 'should no ready event with new wechaty instance');
    await wechaty.start();
    t.ok(spy.notCalled, 'should no ready event right start wechaty started');
    puppet.emit('ready', { data: 'test' });
    t.ok(spy.calledOnce, 'should fire ready event after puppet ready');
    await wechaty.stop();
    await wechaty.start();
    puppet.emit('ready', { data: 'test' });
    t.ok(spy.calledTwice, 'should fire ready event second time after stop/start wechaty');
    await wechaty.stop();
});
(0, tstest_1.test)('ready()', async (t) => {
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const wechaty = new wechaty_base_js_1.WechatyBase({ puppet });
    const sandbox = tstest_1.sinon.createSandbox();
    const spy = sandbox.spy();
    wechaty.ready()
        .then(spy)
        .catch(e => t.fail('rejection: ' + e));
    t.ok(spy.notCalled, 'should not ready with new wechaty instance');
    await wechaty.start();
    t.ok(spy.notCalled, 'should not ready after right start wechaty');
    puppet.emit('ready', { data: 'test' });
    await new Promise(resolve => setImmediate(resolve));
    t.ok(spy.calledOnce, 'should ready after puppet ready');
    await wechaty.stop();
    await wechaty.start();
    wechaty.ready()
        .then(spy)
        .catch(e => t.fail('rejection: ' + e));
    puppet.emit('ready', { data: 'test' });
    await new Promise(resolve => setImmediate(resolve));
    t.ok(spy.calledTwice, 'should ready again after stop/start wechaty');
    await wechaty.stop();
});
(0, tstest_1.test)('on/off event listener management', async (t) => {
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const wechaty = new wechaty_base_js_1.WechatyBase({ puppet });
    const onMessage = (_) => { };
    t.equal(wechaty.listenerCount('message'), 0, 'should no listener after initializing');
    wechaty.on('message', onMessage);
    t.equal(wechaty.listenerCount('message'), 1, 'should +1 listener after on(message)');
    wechaty.off('message', onMessage);
    t.equal(wechaty.listenerCount('message'), 0, 'should -1 listener after off(message)');
});
(0, tstest_1.test)('wrapAsync() async function', async (t) => {
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const wechaty = new wechaty_base_js_1.WechatyBase({ puppet });
    const spy = tstest_1.sinon.spy();
    wechaty.on('error', spy);
    const DATA = 'test';
    const asyncFunc = async () => DATA;
    const syncFunc = wechaty.wrapAsync(asyncFunc);
    t.notOk(syncFunc(), 'should get sync function return void');
    t.ok(spy.notCalled, 'should not emit error when sync function return value');
    const asyncFunc2 = async () => { throw new Error('test'); };
    const syncFunc2 = wechaty.wrapAsync(asyncFunc2);
    t.doesNotThrow(() => syncFunc2(), 'should not throw when async function throw error');
    await wechaty.sleep(0); // wait async event loop task to be executed
    t.ok(spy.calledOnce, 'should emit error when async function throw error');
});
(0, tstest_1.test)('wrapAsync() promise', async (t) => {
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const wechaty = new wechaty_base_js_1.WechatyBase({ puppet });
    const spy = tstest_1.sinon.spy();
    wechaty.on('error', spy);
    const DATA = 'test';
    const promise = Promise.resolve(DATA);
    const wrappedPromise = wechaty.wrapAsync(promise);
    t.equal(await wrappedPromise, undefined, 'should resolve Promise<any> to void');
    const rejection = Promise.reject(new Error('test'));
    const wrappedRejection = wechaty.wrapAsync(rejection);
    t.equal(wrappedRejection, undefined, 'should be void and not to reject');
    t.equal(spy.callCount, 0, 'should have no error before sleep');
    await wechaty.sleep(0); // wait async event loop task to be executed
    t.equal(spy.callCount, 1, 'should emit error when promise reject with error');
});
(0, tstest_1.test)('WechatyBaseProtectedProperty', async (t) => {
    const noOneLeft = true;
    t.ok(noOneLeft, 'should match Wechaty properties for every protected property');
});
(0, tstest_1.test)('WechatySkeleton: super.{start,stop}()', async (t) => {
    const sandbox = tstest_1.sinon.createSandbox();
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const memory = new memory_card_1.MemoryCard();
    const wechaty = new WechatyTest({
        memory,
        puppet,
    });
    const startStub = sandbox.stub(wechaty_skeleton_js_1.WechatySkeleton.prototype, 'start').resolves();
    const stopStub = sandbox.stub(wechaty_skeleton_js_1.WechatySkeleton.prototype, 'stop').resolves();
    t.ok(startStub.notCalled, 'should not called before start');
    t.ok(stopStub.notCalled, 'should not called before stop');
    await wechaty.start();
    t.ok(startStub.calledOnce, 'should call the skeleton start(), which means all mixin start()s are chained correctly');
    t.ok(stopStub.notCalled, 'should not call stop yet');
    await wechaty.stop();
    t.ok(startStub.calledOnce, 'should only call start once');
    t.ok(stopStub.calledOnce, 'should call the skeleton stop(), which means all mixin stops()s are chained correctly');
});
//# sourceMappingURL=wechaty-base.spec.js.map